@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gdk/gdkx.h>
+#include <gio/gio.h>
 
 #include <glib/gi18n-lib.h>
 #define XKEYBOARD_CONFIG_(String) ((char *) g_dgettext ("xkeyboard-config", String))
@@ -84,7 +84,8 @@ struct _GnomeXkbInfoPrivate
   gchar **current_parser_text;
 };
 
-G_DEFINE_TYPE (GnomeXkbInfo, gnome_xkb_info, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_CODE (GnomeXkbInfo, gnome_xkb_info, G_TYPE_OBJECT,
+                         G_ADD_PRIVATE (GnomeXkbInfo));
 
 static void
 free_layout (gpointer data)
@@ -405,6 +406,12 @@ parse_end_element (GMarkupParseContext  *context,
                                                     priv->current_parser_variant->xkb_name,
                                                     NULL);
 
+      if (g_hash_table_contains (priv->layouts_table, priv->current_parser_variant->id))
+        {
+          g_clear_pointer (&priv->current_parser_variant, free_layout);
+          return;
+        }
+
       g_hash_table_replace (priv->layouts_table,
                             priv->current_parser_variant->id,
                             priv->current_parser_variant);
@@ -614,7 +621,7 @@ ensure_rules_are_parsed (GnomeXkbInfo *self)
 static void
 gnome_xkb_info_init (GnomeXkbInfo *self)
 {
-  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GNOME_TYPE_XKB_INFO, GnomeXkbInfoPrivate);
+  self->priv = gnome_xkb_info_get_instance_private (self);
 }
 
 static void
@@ -640,8 +647,6 @@ gnome_xkb_info_class_init (GnomeXkbInfoClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->finalize = gnome_xkb_info_finalize;
-
-  g_type_class_add_private (gobject_class, sizeof (GnomeXkbInfoPrivate));
 }
 
 /**
